//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  events.cpp
//  This is a part of the systemevents source code.
//
//  Copyright (C) 2003 ilmcuts.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include "events.hpp"
#include "main.hpp"
#include <strsafe.h>


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Event Name Table
//
const NamedValue<EventIDs> g_EventTable[] =
{
    // LS Events
    "Start", EVENT_START,
    "Quit", EVENT_QUIT,
    "Refresh", EVENT_REFRESH,

    // Windows Events
    "DisplayChange", EVENT_DISPCHANGE,
    "FontChange", EVENT_FONTCHANGE,
    // "ScreenSaverOn", EVENT_SCREENSAVER_ON,
    // "ScreenSaverOff", EVENT_SCREENSAVER_OFF,
    "Hibernate", EVENT_HIBERNATE,
    "Resume", EVENT_RESUME,
    "Shutdown", EVENT_SHUTDOWN,
    "Logoff", EVENT_LOGOFF,

    // Shell Hook Events
    "FullscreenOn", EVENT_FULLSCREEN_ON,
    "FullscreenOff", EVENT_FULLSCREEN_OFF
};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Globals
//
const size_t g_stEventTableItems = sizeof(g_EventTable)/sizeof(g_EventTable[0]);
EventMap g_EventMap;


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ExecuteEvent
//
void ExecuteEvent(EventIDs uIdentifier)
{
    using std::pair;
    using EventMap::const_iterator;

    pair<const_iterator, const_iterator> range =
        g_EventMap.equal_range(uIdentifier);

    for (const_iterator iter = range.first; iter != range.second; ++iter)
    {
        LSExecute(NULL, iter->second.c_str(), 0);
    }
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// WindowProc
//
LRESULT WINAPI WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch (uMsg)
    {
        case LM_REFRESH:
        {
            ExecuteEvent(EVENT_REFRESH);
            ClearConfig();
            ReadConfig();
        }
        break;

        case LM_WINDOWACTIVATED:
        {
            static bool bIsFullScreen = false;

            if ((lParam > 0) && !bIsFullScreen)
            {
                bIsFullScreen = true;
                ExecuteEvent(EVENT_FULLSCREEN_ON);
            }
            else if ((lParam <= 0) && bIsFullScreen)
            {
                bIsFullScreen = false;
                ExecuteEvent(EVENT_FULLSCREEN_OFF);
            }
        }
        break;

        case WM_DISPLAYCHANGE:
        {
            ExecuteEvent(EVENT_DISPCHANGE);
        }
        break;

        case WM_FONTCHANGE:
        {
            ExecuteEvent(EVENT_FONTCHANGE);
        }
        break;

        case WM_POWERBROADCAST:
        {
            if (wParam == PBT_APMSUSPEND)
            {
                ExecuteEvent(EVENT_HIBERNATE);
            }
            else if (wParam == PBT_APMRESUMESUSPEND)
            {
                ExecuteEvent(EVENT_RESUME);
            }
        }
        break;

        case WM_ENDSESSION:
        {
            if (wParam)
            {
                if (lParam & ENDSESSION_LOGOFF)
                {
                    ExecuteEvent(EVENT_LOGOFF);
                }
                else if (lParam == 0)
                {
                    ExecuteEvent(EVENT_SHUTDOWN);
                }
            }
        }
        break;

        case LM_GETREVID:
        {
            if (wParam == 0 || wParam == 1)
            {
                StringCchCopy((char*)lParam, 64, "SystemEvents 0.1");
                return strlen((const char*)lParam);
            }

            return 0;
        }
        break;

        default:
        break;
    }

    return DefWindowProc(hWnd, uMsg, wParam, lParam);
}
