//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  main.cpp
//  This is a part of the systemevents source code.
//
//  Copyright (C) 2003 ilmcuts.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include "main.hpp"
#include "events.hpp"
#include <strsafe.h>


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Globals
//
ATOM g_aClass = NULL;
HWND g_hMainWnd = NULL;

const UINT lsMessages[] =
{
    LM_GETREVID,
    LM_REFRESH,
    LM_WINDOWACTIVATED,
    NULL
};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Error
//
void Error(const char* pszFormat, ...)
{
    char szMessage[MAX_PATH] = { 0 };
    va_list argList;
    
    va_start(argList, pszFormat);
    StringCchVPrintf(szMessage, MAX_PATH, pszFormat, argList);
    va_end(argList);
    
    MessageBox(NULL, szMessage, "systemevents.dll Error",
        MB_TOPMOST | MB_SETFOREGROUND | MB_ICONERROR);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ReadEventSettings
//
void ReadEventSettings(std::string sConfig, EventIDs uIdentifier)
{
    sConfig.insert(0, "*On");
    
    FILE* fStep = LCOpen(NULL);

    if (!fStep)
    {
        throw std::runtime_error("Could not open step.rc");
    }

    char szBuffer[MAX_LINE_LENGTH] = { 0 };
    
    while (LCReadNextConfig(fStep, sConfig.c_str(), szBuffer, MAX_LINE_LENGTH))
    {
        // skip the first token (*OnBlah)
        LPCSTR pszNextToken = szBuffer;
        GetToken(szBuffer, NULL, &pszNextToken, FALSE);

        if (pszNextToken)
        {
            g_EventMap.insert(EventMap::value_type(uIdentifier, pszNextToken));
        }
    }

    LCClose(fStep);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ReadConfig
//
void ReadConfig()
{
    for (size_t stCounter = 0; stCounter < g_stEventTableItems; ++stCounter)
    {
        ReadEventSettings(
            g_EventTable[stCounter].pszName, g_EventTable[stCounter].tValue);
    }
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ClearConfig
//
void ClearConfig()
{
    g_EventMap.clear();
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// initModuleEx
//
int initModuleEx(HWND hLiteStep, HINSTANCE hInstance, LPCSTR szLSpath)
{
    try
    {
        ReadConfig();

        WNDCLASSEX wc = { 0 };
        wc.cbSize = sizeof(WNDCLASSEX);
        wc.lpfnWndProc = WindowProc;
        wc.hInstance = hInstance;
        wc.lpszClassName = "LSSystemEventsClass";

        g_aClass = RegisterClassEx(&wc);

        if (g_aClass)
        {
            g_hMainWnd = CreateWindowEx(
                WS_EX_NOPARENTNOTIFY | WS_EX_TOOLWINDOW,
                MAKEINTATOM(g_aClass),
                NULL,
                WS_POPUP,
                0, 0, 0, 0,
                NULL, NULL,
                hInstance,
                NULL);

            if (g_hMainWnd)
            {
                SendMessage(GetLitestepWnd(), LM_REGISTERMESSAGE,
                    (WPARAM)g_hMainWnd, (LPARAM)lsMessages);
            }
        }

        if (!g_aClass || !g_hMainWnd)
        {
            Error("Failed to initialize main window");
            quitModule(hInstance);
        }
    }
    catch (std::exception& e)
    {
        Error("Error during initialization: %s", e.what());
        quitModule(hInstance);
    }

    ExecuteEvent(EVENT_START);

    return 0;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// quitModule
//
void quitModule(HINSTANCE hInstance)
{
    try
    {
        ExecuteEvent(EVENT_QUIT);

        ClearConfig();

        if (g_hMainWnd)
        {
            SendMessage(GetLitestepWnd(), LM_UNREGISTERMESSAGE,
                (WPARAM)g_hMainWnd, (LPARAM)lsMessages);

            DestroyWindow(g_hMainWnd);
            g_hMainWnd = NULL;
        }

        if (g_aClass)
        {
            UnregisterClass(MAKEINTATOM(g_aClass), hInstance);
            g_aClass = NULL;
        }
    }
    catch (std::exception& e)
    {
        Error("Error during shutdown: %s", e.what());
    }
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// DllMain
//
// DLL entry point
//
BOOL APIENTRY DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /* pvReserved */)
{
    if (dwReason == DLL_PROCESS_ATTACH)
    {
        DisableThreadLibraryCalls(hInstance);
    }
    
    return TRUE;
}
